//
//  macro.cpp
//  Lily
//
//  Created by KMY on 2015/05/09.
//  Copyright (c) 2015年 KMY. All rights reserved.
//

#include "macro.h"
#include "message.h"

Macro::Macro() : Variable() {
}

Macro::Macro(const lchar* vn, int vt) : Variable(vn, vt) {
	startToken = 0;
	firstParam = 0;
}

Macro::~Macro() {
	delete firstParam;
}

void Macro::setStartToken(const lchar *p) {
	startToken = p;
}

const lchar* Macro::getStartToken() {
	return startToken;
}

Variable* Macro::getFirstParam() {
	return firstParam;
}

void Macro::addParam(const lchar *vn, int vt) {
	addParam(new Variable(vn, vt));
}

void Macro::addParam(const lchar *vn, const lchar* vtn) {
	addParam(new Variable(vn, vtn));
}

void Macro::addParam(Variable *obj) {
	if (!firstParam) {
		firstParam = obj;
	}
	else {
		firstParam->add(obj);
	}
}

bool Macro::checkParam(Variable *param, const lchar* vname) {
	
	Variable *param1 = param, *param2 = firstParam;

	// 関数の名前が合っているか
	if (vname && !Character::equal(vname, name)) return false;
	
	// パラメータの中身が合っているか
	while (param1 && param2) {
		if (!param1->checkVariable(param2, false)) return false;
		//if (!checkVariable(this, false)) return false;
		param1 = param1->next();
		param2 = param2->next();
	}
	
	// 引数の数が合っているか
	if ((!param1 && param2) || (param1 && !param2)) return false;
	
	return true;
}

void Macro::macroTest() {
	char* nc;
	Character::toCharn(name, &nc);
	
	// 関数名
	printf(" -m:%s\t\t", nc);
	
	// パラメータ
	Variable* p = firstParam;
	while (p) {
		p->variableTest();
		p = p->next();
	}
	printf("\n");

	delete nc;
}
